--[[
	Party Debuffs
	By: Luckiee-Aggramar

	Creates a text frame listing all debuffs on all party members including debuff spell name, spell ID, tooltip text, caster name, and stack count.

	Automatically hidden for groups with more than 5 members.

	Reposition frame with left mouse button.  Resize by dragging lower right corner of frame.

	Usage: to turn on / off
		/alldebuffs [show | hide]

--]]


	local _, addon = ...;

	SLASH_ALLDEBUFFS1 = "/alldebuffs"
	SlashCmdList["ALLDEBUFFS"] = function(msg)
		if msg == "show" then
			AD_Panel:Show()
			print("Party Debuffs frame turned on.")
		elseif msg == "hide" then
			AD_Panel:Hide()
			print("Party Debuffs frame turned off.")
		else
			print("Usage: /alldebuffs [show \| hide]")
		end
	end

	-- Saved Variables defaults.
	ADConfig = {
		["FontSize"] = 12,
		["debug"] = false,
		["Locked"] = false,
	}

	local ADunits = {
		'player',
		'party1',
		'party2',
		'party3',
		'party4'
	}

	-- TO DO: put blacklist of debuffs to ignore in table here (heroism exhaustion, 11196=cannot be bandaged again, etc.)

	local ADtypecolors = {
		["Poison"]	= "009900",
		["Magic"]	= "3399ff",
		["Curse"]	= "bb55ff",	--"9900ff",
		["Disease"]	= "ce8e0e", --"996600",
		["none"]		= "aaaaaa"
	}

	local ADignorelist = {
		-- lust/hero, etc.
		[57723]=1,	-- exhaustion
		[57724]=1,	-- sated
		[80354]=1,	-- temporal displacement
		[95809]=1,	-- instanity
		[264689]=1,	-- fatigued
		[279737]=1,	-- prepare-for-battle
		[206151]=1,	-- challenger's burden (keystone)
		[186263]=1,	-- disc. priest, shadow mend
		--...
	}

	-- indent string used in front of each debuff line (helps distinguish debuff lines from party names when debuff type is none, i.e. white)
	local ADindent = ""; --"   "

	-- string to prefix on each unit name, typically as alternative to extra line spacing, for better readability while remaining compact
	local ADnameIndent = " --- "; --"   "

	-- string (extra lines) to add to end of each unit name for better readability
	local ADnameLineSpacing = "\n"; --"\n\n"






--------------------
function debugprint(msg)
	local line = strmatch(debugstack(), ":(%d):");
	print("PartyDebuffs line ".. line ..": ".. msg);
end

--------------------
function inSmallParty()
	-- return true if solo or in a party of 5 or fewer
	local n = GetNumGroupMembers()
	local isSmall = false

	if n < 6 then
		isSmall = true
	end

--	if ADConfig.debug then print("Small party? ".. tostring(isSmall) .."("..n..")" ) end

	return isSmall
end

--------------------
function createADFrame() 

	if ADConfig.debug then print("createADFrame()") end

	AD_Panel = CreateFrame("Frame", "AD_TipFrame", UIParent)
	AD_Panel:SetFrameStrata("MEDIUM")
	AD_Panel:SetWidth(450)
	AD_Panel:SetHeight(175)
	AD_Panel:SetClampedToScreen(true)

	-- 20180901: resize main frame, not header bar
	AD_Panel:EnableMouse(true)
	AD_Panel:SetMovable(true)
	AD_Panel:SetResizable(true)
	AD_Panel:SetMinResize(200,100)
	AD_Panel:RegisterForDrag("LeftButton", "RightButton")
--	AD_Panel:SetScript("OnDragStart", function(self) self:StartMoving() end)
	AD_Panel:SetScript("OnDragStart", function(self, button)
		-- don't allow drag if configuration is set to Locked
		if not ADConfig.Locked then
			self:StartMoving()
		end
	end)
	AD_Panel:SetScript("OnDragStop", function(self)
		self:StopMovingOrSizing()
		-- set text to fit new panel size
		AD_DebuffText:SetWidth( AD_Panel:GetWidth() )
	end)


	AD_PanelTexture = AD_Panel:CreateTexture(nil, "Background")
	AD_PanelTexture:ClearAllPoints()
	AD_PanelTexture:SetColorTexture(0,0,0, 0.65)
	AD_PanelTexture:SetAllPoints(AD_Panel)

	AD_Panel:SetPoint("CENTER", UIParent)
	AD_Panel:Show()
	if ADConfig.debug then print("AD_Panel:Show() "..debugstack()) end


	-- Frame Tip Text
	AD_DebuffText = AD_Panel:CreateFontString("AD_DebuffText", nil, GameFontNormal)
	AD_DebuffText:SetPoint("TOPLEFT", AD_Panel, "TOPLEFT", 3, -1)
	--AD_DebuffText:SetFont("Fonts\\ARIALN.ttf", 14, nil)
	AD_DebuffText:SetFontObject(GameFontWhite);
	local p,_,_ = AD_DebuffText:GetFont();
	AD_DebuffText:SetFont(p, ADConfig['FontSize'], nil)
--	AD_DebuffText:SetWidth(445)
	AD_DebuffText:SetWidth( AD_Panel:GetWidth() )
	AD_DebuffText:SetJustifyH("LEFT")
	AD_DebuffText:SetJustifyV("TOP")
	AD_DebuffText:SetText("Party Debuffs")	-- title
	AD_DebuffText:SetShadowOffset(.5, -.5)

	-- config button
	AD_ConfigBtn = CreateFrame("Button", "AD_ConfigButton", AD_Panel)
	AD_ConfigBtn:SetFrameLevel(5)
	AD_ConfigBtn:ClearAllPoints()
	AD_ConfigBtn:SetHeight(16)
	AD_ConfigBtn:SetWidth(16)
	AD_ConfigBtn:SetNormalTexture("Interface\\Buttons\\UI-OptionsButton")
	AD_ConfigBtn:SetHighlightTexture("Interface\\Buttons\\UI-OptionsButton", 1.0)
	AD_ConfigBtn:SetAlpha(0.45)
	AD_ConfigBtn:SetPoint("TOPRIGHT", AD_Panel, "TOPRIGHT", -4, -2)

	AD_ConfigBtn:RegisterForClicks("LeftButtonUp", "RightButtonUp")
	AD_ConfigBtn:SetScript("OnClick", function()
		AllDebuffs_ConfigFrame:Show()
	end)
	AD_ConfigBtn:Show()



	AD_ResizeObj = CreateFrame("Button", "AD_ResizeButton", AD_Panel)
	AD_ResizeObj:SetFrameLevel(5)
	AD_ResizeObj:ClearAllPoints()
	AD_ResizeObj:SetHeight(16)
	AD_ResizeObj:SetWidth(16)
	AD_ResizeObj:SetNormalTexture("Interface\\CHATFRAME\\UI-ChatIM-SizeGrabber-Up")
	AD_ResizeObj:SetHighlightTexture("Interface\\CHATFRAME\\UI-ChatIM-SizeGrabber-Down", 1.0)
--	AD_ResizeObj:SetAlpha(0.5)
	AD_ResizeObj:SetPoint("BOTTOMRIGHT", AD_Panel, "BOTTOMRIGHT", 0, 0)

	AD_ResizeObj:RegisterForClicks("LeftButton", "RightButton")
	AD_ResizeObj:SetScript("OnMouseDown", function(self, button)
		-- don't allow drag if configuration is set to Locked
		if not ADConfig.Locked then
			AD_Panel:StartSizing()
		end
	end)
	AD_ResizeObj:SetScript("OnMouseUp", function(self)
		AD_Panel:StopMovingOrSizing()
		-- set text to fit new panel size
		AD_DebuffText:SetWidth( AD_Panel:GetWidth() )
--		AD_DebuffText:SetHeight( AD_Panel:GetHeight() )
	end)
	AD_ResizeObj:Show()




	
	AD_Panel:RegisterEvent("UNIT_AURA")
	AD_Panel:RegisterEvent("PET_BATTLE_OPENING_START")
	AD_Panel:RegisterEvent("PET_BATTLE_CLOSE")
	AD_Panel:RegisterEvent("PLAYER_ENTER_COMBAT")
	AD_Panel:RegisterEvent("GROUP_ROSTER_UPDATE")
	AD_Panel:RegisterEvent("UNIT_ENTERED_VEHICLE")
	AD_Panel:RegisterEvent("UNIT_EXITED_VEHICLE")
	AD_Panel:RegisterEvent("ADDON_LOADED")


	AD_Panel:SetScript("OnEvent", function(self, event, arg3, ...)

		if (event == "PET_BATTLE_OPENING_START") or (event == "UNIT_ENTERED_VEHICLE") then

			AD_Panel:Hide()

			if ADConfig.debug then print("AD_Panel:Hide() PET_BATTLE_OPENING_START or UNIT_ENTERED_VEHICLE") end


		elseif (event == "PET_BATTLE_CLOSE") or (event == "UNIT_EXITED_VEHICLE") then

			if inSmallParty() then
				AD_Panel:Show()
			end

			if ADConfig.debug then print("AD_Panel:Show() PET_BATTLE_CLOSE or UNIT_EXITED_VEHICLE") end

		elseif event == "PLAYER_ENTER_COMBAT" then

			-- if we're in a raid (not party), hide frames
			if not inSmallParty() then
				AD_Panel:Hide()
				if ADConfig.debug then print("AD_Panel:Hide() PLAYER_ENTER_COMBAT") end
			end

		elseif event == "GROUP_ROSTER_UPDATE" then

			-- if party members have changed (gone solo, joined party, left party), hide or show frame appropriately
			if inSmallParty() then
--				print("Showing Party Debuffs for non-raid group.")
				AD_Panel:Show()
				if ADConfig.debug then print("AD_Panel:Show() GROUP_ROSTER_UPDATE inSmallParty") end
			else
--				print("Hiding Party Debuffs in raid group.")
				AD_Panel:Hide()
				if ADConfig.debug then print("AD_Panel:Hide() GROUP_ROSTER_UPDATE *not* inSmallParty") end
			end

		elseif event == "ADDON_LOADED" then

			if arg3 ~= "PartyDebuffs" then return end

			if ADConfig.debug then print("PartyDebuffs ADDON_LOADED") end

			-- load saved variables

			if ADConfig.Locked == nil then ADConfig.Locked = false end
			AllDebuffs_Locked_CheckBox:SetChecked(ADConfig.Locked)
			if ADConfig.Locked then
				AD_ResizeObj:Hide()
			end

			-- default fontsize if it didn't exist before
--			print(ADConfig.FontSize)
			ADConfig.FontSize = ADConfig.FontSize or 12

			-- update display to latest font size setting
			local p,_,_ = AD_DebuffText:GetFont();
			AD_DebuffText:SetFont(p, ADConfig['FontSize'], nil)

			-- also update config window's font size value
			AllDebuffs_FontSize_Value:SetText(ADConfig['FontSize'])


		elseif event == "UNIT_AURA" then

			-- if the frame is not currently shown, do nothing.
			if not AD_Panel:IsShown() then return end

			-- if we're in a raid make sure we don't do any processing of debuffs
			if not inSmallParty() then return end

			-- initialize text with a title line
			local text = "Party Debuffs\n"

			local process_sid = true
			local castername = ""
			local selfcast = false

			-- create a blank tooltip placeholder
			local tip = AD_MyTT or CreateFrame("GameTooltip","AD_MyTT", nil, "GameTooltipTemplate")
			tip:SetOwner(UIParent, "ANCHOR_NONE")

			--[[
			So the display of party debuffs is consistent no matter what debuff is added/dropped, we'll display ALL debuffs every time, always in the same order.
			--]]
			for c,unit in ipairs(ADunits) do

				if (UnitExists(unit)) then

					-- put player's name on new line
					text = text .. ADnameIndent .. strupper(UnitName(unit)) .. ADnameLineSpacing;

					-- keep track of all debuffs for this unit, so we don't display any more than once.
					local sids_this_unit = {}

					-- check for up to 8 debuffs
					for i=1,8 do

						-- name, icon, count, debuffType, duration, expirationTime, unitCaster, isStealable, nameplateShowPersonal, spellId, canApplyAura, isBossDebuff, nameplateShowAll, timeMod, value1, value2, value3 = UnitDebuff("unit", index[, "filter"]])

						local spellname,ico,stacks,t,d,e,caster,_,_,sid = UnitDebuff(unit,i)

						-- if we've seen this debuff before for this unit, ignore it (don't process it)
						process_sid = true
						if sid then
							local test = sids_this_unit[sid] or false
--							print("sid already exists? ".. sid ..' '..(test or ""))
							if sids_this_unit[sid] then
--								print("Ignoring duplicate "..sid)
								process_sid = false
							end
						end

						-- see if it's a self-applied debuff (we'll probably want to ignore these)
						-- also get name of unit that cast debuff
						selfcast = false
						if caster then
							-- see if it's self-cast
							if caster == unit then
								selfcast = true
							end
							castername = " (".. UnitName(caster) ..") "
						else
							castername = ""
						end
					--	print(caster.." - "..castername)


						-- if the debuff exists, and this unit doesn't already have that debuff, and it's not in our blacklist, and it's not a self-applied debuff, then include it in the list to be displayed.
						if sid and process_sid and (not ADignorelist[sid]) and (not selfcast) then

							-- add this sid to the list of debuffs for this player
							sids_this_unit[sid] = 1
--							print("adding "..sid.." to sid list")


							-----------------------------------------------------
							-- READING TOOLTIP METHOD...
							tip:SetUnitDebuff(unit,i)
							local descline1 = AD_MyTTTextLeft2:GetText() or ""

							local color = ADtypecolors[t] or "aaaaaa"

							if stacks <= 1 then stacks = "" end

							-- add stack count in white and end string by resetting color back to debuff type color.
							stacks = " |cffffffff"..stacks.."|cff" .. color

							text = text .. ADindent .. "|T".. ico ..":0|t" .. "|cff" .. color .. " " .. spellname .." (".. sid .."): ".. descline1 .. "|cff"..ADtypecolors["none"] .. castername .. stacks .. "|r" .. "\n"
							-----------------------------------------------------


						end	-- end of sid
					end	-- end of for each debuff

					-- clear list of debuffs for the current unit
					for id,_ in pairs(sids_this_unit) do
					--	print("Nulling ".. id)
						sids_this_unit[id] = nil
					end

				end	-- end of UnitExists() check

				-- add a blank line to separate this player from the next.
			--	text = text .. "\n";

			end	-- end of for each unit

			-- update the alldebuffs frame
			AD_DebuffText:SetText(text)

		end	-- end of UNIT_AURA

	end)	-- end of OnEvent function


end	-- end of createADFrame()

--------------------
function AllDebuffs_SetFontSize_Up()
	if ADConfig['FontSize'] < 30 then ADConfig['FontSize'] = ADConfig['FontSize'] + 1 end
	AllDebuffs_UpdateFontSize()
end

--------------------
function AllDebuffs_SetFontSize_Down()
	if ADConfig['FontSize'] > 8 then ADConfig['FontSize'] = ADConfig['FontSize'] - 1 end
	AllDebuffs_UpdateFontSize()
end

--------------------
function AllDebuffs_UpdateFontSize()
	-- update display
	local p,_,_ = AD_DebuffText:GetFont();
	AD_DebuffText:SetFont(p, ADConfig['FontSize'], nil)
	AllDebuffs_FontSize_Value:SetText(ADConfig['FontSize'])
end

--------------------
function AllDebuffsFrameInit()
	-- TESTING
end


--------------------
function AllDebuffs_Locked_CheckBox_Toggle()

	local isChecked = AllDebuffs_Locked_CheckBox:GetChecked()

	if isChecked then
		if ADConfig.debug then print("Locking PartyDebuffs frame.") end
		ADConfig.Locked = true
		AD_ResizeObj:Hide()
	else
		if ADConfig.debug then print("Unlocking PartyDebuffs frame.") end
		ADConfig.Locked = false
		AD_ResizeObj:Show()
	end
end


--------------------
createADFrame()

--------------------
